<?php
namespace MyMenu\Controllers;

use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\FilterCriteria;
use MyMenu\Libraries\KeyValueStorage;

class Pesquisa {

    private $crudService;
    private $environment;
    private $session;
    private $newCode;

	public function __construct($crudService, $environment, $session, $newCode, $entityManager) {
        $this->crudService = $crudService;
        $this->environment = $environment;
        $this->session = $session;
        $this->newCode = $newCode;
        $this->entityManager = $entityManager;
    }
    
    public function findProgramacaopesq(Request\Filter $request, Response $response) {
        try {
            $this->dataSourceName = 'programacaopesq_mymenu';
            
            $isAnon = KeyValueStorage::get("isAnon");
            $usr = $isAnon && $isAnon === true ? null : KeyValueStorage::get('NRUSUARIOAPP');

            $filterCriteria = $request->getFilterCriteria();
            $filterCriteria->addCondition('NRLOCALPESQ', KeyValueStorage::get('NRLOCALPESQ'));
            $filterCriteria->addCondition('CDFILIAL', KeyValueStorage::get('CDFILIAL'));
            $filterCriteria->addCondition('NRUSUARIO', $usr);
            
            $filterCriteria->setDataSourceName($this->dataSourceName);
            $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
            $newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, array());
            $this->crudService->find($response, $newFilterCriteria);
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function pesquisaSubmmit(Request\Row $request, Response $response) {

        $row = $request->getRow();
        if(!isset($row['surveyData'])){ // keeping legacy compatibility
            $this->legacyPesquisaSubmit($request, $response);
            return;
        }
        $execDataSourceName = 'execucaopesq';
        $anwsDataSourceName = 'respostapesq';
        
        $anwsRows = array();
        
        $nrExecucaopesq = $this->newCode->getCodeByTableName("EXECUCAOPESQ", $this->environment->getNrorg());
        $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        
        $execRow = array(
            '__is_new' => true,
            'NREXECUCAOPESQ' => $nrExecucaopesq,
            'CDFILIAL' => KeyValueStorage::get('CDFILIAL'),
            'NRLOCALPESQ' => KeyValueStorage::get('NRLOCALPESQ'),
            'NRPROGRAMACAOPESQ' => $row['surveyData']['NRPROGRAMACAOPESQ'],
            'NRQUESTIONARIO' => $row['surveyData']['NRQUESTIONARIO'],
            'IDAPLICATIVO' => 'MYMENU',
            'NRUSUARIOPESQ' => KeyValueStorage::get('NRUSUARIOAPP'),
            'CDOPERADOR' => KeyValueStorage::get('NRUSUARIOAPP'),
            'DTHRINICIOPESQ' => $currentTime,
            'DTHRFIMPESQ' => $currentTime
        );
        foreach ($row as $rowKey=>$rowValue) {
            if (strpos($rowKey, 'QUESTAO') === 0) {
                $nrQuestao = substr($rowKey, 7);
                if(strpos($nrQuestao, "_") !== false) {
                    $nrQuestao = str_replace("QUESTAO", "", $nrQuestao);
                    $splitedQuestionName = explode("_", $nrQuestao);
                    $arrayLength = sizeof($splitedQuestionName);
                    $nrQuestao = $splitedQuestionName[0];
                }
                if (is_array($rowValue) && !array_key_exists(0, $rowValue) ) { // if isn't numeric array
                    $anwsRow = array(
                        '__is_new' => true,
                        'NRRESPOSTAPESQ' => $this->newCode->getCodeByTableName("RESPOSTAPESQ", $this->environment->getNrorg()),
                        'NREXECUCAOPESQ' => $nrExecucaopesq,
                        'NRQUESTAO' => $nrQuestao,
                        'NRRESPOSTA' => $rowValue['value'],
                        'TXRESPOSTAPESQ' => $rowValue['description'],
                        'NRRESPOSTAANTERIOR' => $rowValue['parentAnswer']
                    );
                    array_push($anwsRows, $anwsRow);
                } else if(is_array($rowValue)) {
                    foreach ($rowValue as $anwsValue) {
                        $anwsRow = array(
                            '__is_new' => true,
                            'NRRESPOSTAPESQ' => $this->newCode->getCodeByTableName("RESPOSTAPESQ", $this->environment->getNrorg()),
                            'NREXECUCAOPESQ' => $nrExecucaopesq,
                            'NRQUESTAO' => $nrQuestao,
                            'NRRESPOSTA' => $anwsValue['value'],
                            'TXRESPOSTAPESQ' => $anwsValue['description'],
                            'NRRESPOSTAANTERIOR' => $anwsValue['parentAnswer']
                        );
                        array_push($anwsRows, $anwsRow);
                    }
                }
            }
        }

        $execRequest = new Request\Row($execRow, $request->getRoutePath(), $request->getMethod(), $request->getUserId());
        $this->save($execRequest, $execDataSourceName);
        
        $anwsDataSet = new DataSet($anwsDataSourceName, $anwsRows);
        $anwsRequest = new Request\DataSet($anwsDataSet, $request->getRoutePath(), $request->getMethod(), $request->getUserId());
        $this->save($anwsRequest, $anwsDataSourceName);
    }
    
    /**
     * TODO: remove in next version
     */
    private function legacyPesquisaSubmit(Request\Row $request, Response $response) {
        $execDataSourceName = 'execucaopesq';
        $anwsDataSourceName = 'respostapesq';
        
        $row = $request->getRow();
        $anwsRows = array();
        
        $nrExecucaopesq = $this->newCode->getCodeByTableName("EXECUCAOPESQ", $this->environment->getNrorg());
        $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        
        $execRow = array(
            '__is_new' => true,
            'NREXECUCAOPESQ' => $nrExecucaopesq,
            'CDFILIAL' => KeyValueStorage::get('CDFILIAL'),
            'NRLOCALPESQ' => KeyValueStorage::get('NRLOCALPESQ'),
            'NRPROGRAMACAOPESQ' => $row['NRPROGRAMACAOPESQ'],
            'NRQUESTIONARIO' => $row['NRQUESTIONARIO'],
            'IDAPLICATIVO' => 'MYMENU',
            'NRUSUARIOPESQ' => KeyValueStorage::get('NRUSUARIOAPP'),
            'CDOPERADOR' => KeyValueStorage::get('NRUSUARIOAPP'),
            'DTHRINICIOPESQ' => $currentTime,
            'DTHRFIMPESQ' => $currentTime
        );
        
        foreach ($row as $rowKey=>$rowValue) {
            if (strpos($rowKey, 'QUESTAO') === 0) {
                $nrQuestao = substr($rowKey, 7);
                if(strpos($nrQuestao, "_") !== false) {
                    $nrQuestao = substr($nrQuestao, 0, strpos($nrQuestao, "_"));
                }
                if (is_array($rowValue)) {
                    if(isset($rowValue['type']) && $rowValue['type'] === 'open'){
                        $anwsRow = array(
                            '__is_new' => true,
                            'NRRESPOSTAPESQ' => $this->newCode->getCodeByTableName("RESPOSTAPESQ", $this->environment->getNrorg()),
                            'NREXECUCAOPESQ' => $nrExecucaopesq,
                            'NRQUESTAO' => $nrQuestao,
                            'NRRESPOSTA' => null,
                            'TXRESPOSTAPESQ' => $rowValue['value'],
                            'NRRESPOSTAANTERIOR' => null
                        );
                        array_push($anwsRows, $anwsRow);
                    } else {
                        foreach ($rowValue as $anwsValue) {
                            $anwsRow = array(
                                '__is_new' => true,
                                'NRRESPOSTAPESQ' => $this->newCode->getCodeByTableName("RESPOSTAPESQ", $this->environment->getNrorg()),
                                'NREXECUCAOPESQ' => $nrExecucaopesq,
                                'NRQUESTAO' => $nrQuestao,
                                'NRRESPOSTA' => $anwsValue,
                                'TXRESPOSTAPESQ' => $anwsValue,
                                'NRRESPOSTAANTERIOR' => null
                            );
                            array_push($anwsRows, $anwsRow);
                        }
                    }
                } else {
                    $anwsRow = array(
                        '__is_new' => true,
                        'NRRESPOSTAPESQ' => $this->newCode->getCodeByTableName("RESPOSTAPESQ", $this->environment->getNrorg()),
                        'NREXECUCAOPESQ' => $nrExecucaopesq,
                        'NRQUESTAO' => $nrQuestao,
                        'NRRESPOSTA' => $rowValue,
                        'TXRESPOSTAPESQ' => $rowValue,
                        'NRRESPOSTAANTERIOR' => null
                    );
                    array_push($anwsRows, $anwsRow);
                }
            }
        }

        $execRequest = new Request\Row($execRow, $request->getRoutePath(), $request->getMethod(), $request->getUserId());
        $this->save($execRequest, $execDataSourceName);
        // die;
        
        $anwsDataSet = new DataSet($anwsDataSourceName, $anwsRows);
        $anwsRequest = new Request\DataSet($anwsDataSet, $request->getRoutePath(), $request->getMethod(), $request->getUserId());
        $this->save($anwsRequest, $anwsDataSourceName);
    }

    private function save ($request, $dataSourceName){
        $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
        $this->crudService->save($dataSourceName, $rowsToSave);
    }

    public function getNextQuestion(Request $request, Response $response){
        $src_id = $request->getRow()["qstid"];
       $conn = $this->entityManager->getConnection();
       $statement = $conn->prepare("
                  SELECT
                     *
                   FROM
                     QUESTAO  Q
                   WHERE
                       Q.NRQUESTAO = ".$src_id."
                        AND Q.IDATIVO = 'S'
               ");
       $statement->execute();
       $qst_for_screen = $statement->fetchAll();

       $idqst_SQL = $qst_for_screen[0]["NRQUESTAO"];
       $txtqst_sql = $qst_for_screen[0]["TXQUESTAO"];

       $conn = $this->entityManager->getConnection();
       $statement = $conn->prepare("
                  SELECT
                       R.NRRESPOSTA,
                       R.DSRESPOSTA,
                       R.NRQUESTAO,
                       R.NRPROXIMAQUESTAO
                   FROM
                      RESPOSTA R,
                      QUESTAO Q
                   WHERE
                       R.NRQUESTAO = ".$src_id."
                       AND Q.NRQUESTAO = R.NRQUESTAO
                       AND Q.IDATIVO = 'S'
               ");
       $statement->execute();
       $asw_for_screen = $statement->fetchAll();
       $response->addDataSet(new DataSet("qst_especifica", $qst_for_screen[0]));
       $response->addDataSet(new DataSet("resp_especifica", $asw_for_screen));
    }
}