function QRCodeController(ApplicationContext) {
    var ScreenService = ApplicationContext.ScreenService;
    var templateManager = ApplicationContext.templateManager;
    var General = ApplicationContext.General;
    var ZHPromise = ApplicationContext.ZHPromise;
    let FCMBootstrap = ApplicationContext.FCMBootstrap;
    let engine = ApplicationContext.engine;
    var MessageService = ApplicationContext.MessageService;


    var self = this;

    let TColorToRGB = function (TColor) {
        var r = TColor & 0xFF;
        var g = (TColor >> 8) & 0xFF;
        var b = (TColor >> 16) & 0xFF;
        return '(' + r + ',' + g + ',' + b + ')';
    };

    this.openQRCodeInputPopup = function (container) {
        var qrcodeInputPopup = container.getWidget('qrcodeInputPopup');
        qrcodeInputPopup.newRow();
        ScreenService.openPopup(qrcodeInputPopup);
    };

    this.readQRCodeFromInputPopup = function (qrcodeInputPopup) {
        var qrCodeData = qrcodeInputPopup.currentRow.qrCodeData;

        self.readQRCode(qrCodeData).then(function () {
            ScreenService.closePopup(qrcodeInputPopup);
        });
    };

    const updateTOSURL = (httpRequestResponse) => {
        const tosURL = httpRequestResponse.dataset.tosURL && httpRequestResponse.dataset.tosURL[0];
        if(tosURL){
            setLocalVar('tosURL', tosURL);
        }
    };

    let _setNRORGONSESSION = (qrCodeDataRow) => {
        return General.callRestEngine('/setNRORGONSESSION', 'Row', qrCodeDataRow).then((response) => {
            if(response && response.dataset){
                const token = response.dataset.JWT[0];
                setLocalVar('token', token);
            }
        });
    };
    

    let _setLocalVarsFromQRCodeData = (qrCodeDataRow, url, porta, isApp) => {
        return General.callRestEngine('/readQRCode', 'Row', qrCodeDataRow).then(function (response) {
                let qrCodeResp = response.dataset.qrcode;
                let fileServerURL = response.dataset.fileServerURL[0];
                localStorage.setItem('fileServerURL', fileServerURL);
                window.fileServerPath = fileServerURL;
                qrCodeResp.IDBLOQRESERVAANONIMO = qrCodeResp.IDBLOQRESERVAANONIMO == null ? "N" : qrCodeResp.IDBLOQRESERVAANONIMO;
                
                setLocalVar('IDPERANON', qrCodeResp.IDPERANON);
                setLocalVar('IDBLOQRESERVAANONIMO', qrCodeResp.IDBLOQRESERVAANONIMO);
                setLocalVar('IDEXIBENUTR', qrCodeResp.IDEXIBENUTR);
                setLocalVar('IDTAMCARDIMG', qrCodeResp.IDTAMCARDIMG);
                setLocalVar('DSCORPRIMARIA', qrCodeResp.DSCORPRIMARIA || '#E63635');
                if (qrCodeResp.DSCORPRIMARIA) {
                    document.documentElement.style.setProperty('--secundaryColor', 'rgb' + TColorToRGB(qrCodeResp.DSCORPRIMARIA));
                } else {
                    document.documentElement.style.setProperty('--secundaryColor', getLocalVar('DSCORPRIMARIA'));
                }

                setLocalVar('qrCodeURL', qrCodeResp.URLLOGO);
                setLocalVar('NRLOCALPESQ', qrCodeResp.NRLOCALPESQ);
                setLocalVar('DSNOMEEXIBICAO', qrCodeResp.DSNOMEEXIBICAO);
                setLocalVar('customerID', qrCodeResp.CUSTOMERID);
                setLocalVar('validQRCode', true);
                setLocalVar('backendURL', url);


                if(qrCodeResp.DTEXPIRACAO){
                    let dataExpiracaoFormatada = qrCodeResp.DTEXPIRACAO.substring(0,17);
                    dataExpiracaoFormatada  = dataExpiracaoFormatada + "59";
                    setLocalVar('DTEXPIRACAO', dataExpiracaoFormatada);
                }else{
                    
                    let dataExpiracao = new Date();
                    
                    if(qrCodeResp.QTDIASVALIDADE == 0){
                    
                        dataExpiracao = moment(dataExpiracao).format("DD/MM/YYYY 23:59:59");
                        setLocalVar('DTEXPIRACAO', dataExpiracao);
                        
                    }else{
                        
                        dataExpiracao = moment(dataExpiracao).add(qrCodeResp.QTDIASVALIDADE, "days");
                        dataExpiracao = moment(dataExpiracao).format("DD/MM/YYYY HH:mm:59");
                        setLocalVar('DTEXPIRACAO', dataExpiracao);
                    }
                }
                
  
                const token = response.dataset.JWT[0];
                setLocalVar('token', token);
                updateTOSURL(response);
                
                if(isApp) {
                    ScreenService.openWindow('product#login');
                }
        }, function(response){
            if(response.error && response.error.indexOf('MSG_') >= 0)
                MessageService.showMessageByCode(response.error, null, {type: 'error'});
        });
    };

    let _onQRCodeValidationError = (response) => {
        return new Promise((resolve, reject) => {
            setLocalVar('validQRCode', false);
            var errorMessage = 'MSG_ERRO_VALID_QRCODE';
            if (response.data && response.data.error) {
                var errorData = response.data.error;
                errorMessage = errorData;
            } else if(typeof response.error == "string" && response.error.includes('MSG_')) {
                errorMessage = response.error;
            }
            
            MessageService.showMessageByCode(errorMessage, null, {type: 'error'});

            reject(errorMessage);
        });
    };
    
    let loginAnonimous = () => {
        General.callRestEngine("/anonimousAccount", "Row", {}).then(function (response) {
            if(response.dataset){
                const token = response.dataset.JWT[0];
                setLocalVar('token', token);
            }
            setLocalVar('userData', response.dataset.user);
            let fcmToken = localStorage.getItem('fcmToken');
            engine.sendFCMTokenSaveRequest(fcmToken, true).catch(() => engine.sendFCMTokenSaveRequest(fcmToken, false));
            ScreenService.openWindow('product#home');
        }, function (response) {
            var errorMsg = "Erro";
            if (response.data && response.data.error) {
                errorMsg = response.data.error;
            }

            ScreenService.showMessage(ScreenService.i18n(errorMsg), 'E');
        });
    };
    
    this.readQRCode = function (qrCodeData) {
        // let promise = ZHPromise.defer();
        let dados = [];
        
        if(qrCodeData.search("params") > 0) {
            dados = qrCodeData.split('/mobile/#/product#login?params=');
            return this.qrCodeWeb(dados).then(() => {
                loginAnonimous();
            });
        } else {
            dados = qrCodeData.split(';');
            return qrCodeApp(dados);
        }
    };
    
    let qrCodeApp = (dados) => {
        if (dados.length !== 4) {
            _showErrorMessageCode();
        } else {
            let dns = dados[0];
            const porta = dados[1];
            let nrqrcode = dados[2];
            let nrorg = dados[3];
            
            return _setDataAfterQrCode(dns, nrqrcode, nrorg, porta, true);
        }
    };
    
    this.qrCodeWeb = function(dados) {
      if(dados.length !== 2) {
        _showErrorMessageCode();
      } else {
        let dns = dados[0];
        let cript = dados[1];
        
        return _getDecriptedUrlData(cript).then((decriptedData) => {
            if(decriptedData.length > 0) {
                let porta = decriptedData[0].DSPORTA;
                let nrqrcode = decriptedData[0].NRQRCODELOCALPESQ;
                let nrorg = decriptedData[0].NRORG;
                return _setDataAfterQrCode(dns, nrqrcode, nrorg, porta, false);    
            } else {
                _showErrorMessageCode();
            }
        }).catch(_showErrorMessageCode);
      } 
    };
    
    let _showErrorMessageCode = ( errorMsg ) => {
        if( typeof errorMsg !== 'string' || !errorMsg.includes('MSG_') ) { // Message isn't displayed into _onQRCodeValidationError
            MessageService.showMessageByCode('MSG_INVALID_QRCODE', null, {type: 'error'});
        }
        return Promise.reject();
    };

    function getUrlByProtocol(url, protocolToSet = 'http') {
        let urlHTTP = url.replace('https://', 'http://');
        let urlHTTPS = url.replace('http://', 'https://');
        if( protocolToSet === 'http') {
            return urlHTTP;
        } else {
            return urlHTTPS;
        }
    }
    
    let _setDataAfterQrCode = (dns, nrqrcode, nrorg, porta, isApp) => {
        let qrCodeDataRow = {
            'nrqrcode': nrqrcode,
            'nrorg': nrorg
        };

        setLocalVar('qrCodeData', qrCodeDataRow);
        setLocalVar('NRORG', nrorg);

        let url = dns + window.qrcodebaseURL;
        url = getUrlByProtocol(url);
        let urlHTTPS = getUrlByProtocol(url, 'https');
        templateManager.updateURL(url);

        return new Promise((resolve, reject) => {
            _setNRORGONSESSION(qrCodeDataRow).then(() => {
                _setLocalVarsFromQRCodeData(qrCodeDataRow, url, porta, isApp)
                    .then(() => resolve())
                    .catch((errorArg) => {
                        _onQRCodeValidationError(errorArg).catch(error => reject(error));
                    });
            }).catch(() => {
                templateManager.updateURL(urlHTTPS);
                _setNRORGONSESSION(qrCodeDataRow).then(() => {
                    _setLocalVarsFromQRCodeData(qrCodeDataRow, urlHTTPS, porta, isApp);
                    resolve();
                }).catch((errorArg) => {
                    _onQRCodeValidationError(errorArg).catch(error => reject(error));
                });
            });
        });
    };
    
    let _getDecriptedUrlData = (cript) => {
        return General.callRestEngine('/decriptUrlData', 'Row', { 'criptedData': cript }).then((response) => {
            if(response && response.dataset){
                return response.dataset.decriptedData;
            }
        });
    };

    // let bindVisibilityAccessFunctions = function bindVisibilityAccessFunctions() {
    //     Object.defineProperty($scope.widget, 'isVisible', {
    //         set: (newValue) => {
    //             this._isVisible = newValue;
    //         },
    //         get: () => this._isVisible,
    //         // enumerable: true,
    //         // configurable: true
    //     });
    // };
    // // constructor(){
    // // this.$scope = $scope;
    // this._isVisible = $scope.widget._isVisible;
    // bindVisibilityAccessFunctions();
    // }

    let startScan = () =>
        new Promise((resolve, reject) => {
            if ('QRScanner' in window) {
                QRScanner.prepare(() => {
                    QRScanner.scan((err, text) => {
                        if (err) {
                            reject(err);
                        } else {
                            resolve(text);
                        }
                        endScan();
                    });
                    QRScanner.show();
                });
            } else {
                reject('QRScanner is not avaliable');
            }
        });


    let endScan = () => {
        if (QRScanner) {
            // webView.isViisible = true;
            QRScanner.destroy();
        }
    };

    let requestPermission = () =>
        new Promise((resolve, reject) => {
            if (QRScanner) {
                QRScanner.prepare((err, status) => {
                    if (err) {
                        reject(err);
                    }
                    if (status.authorized) {
                        resolve(true);
                    } else if (status.denied) {
                        resolve(false);
                    } else {
                        resolve(false);
                        // needs better support
                        // we didn't get permission, but we didn't get permanently denied. (On
                        // Android, a denial isn't permanent unless the user checks the "Don't
                        // ask again" box.) We can ask again at the next relevant opportunity.
                    }
                });
            } else {
                reject('QRScanner is not avaliable');
            }
        });

    this.backAndCancelEvent = function (widget) {
        widget.isScaning = false;
        document.body.style.background = '';
        if ('QRScanner' in window) {
            QRScanner.getStatus((status) => {
                if (status.scanning && status.showing) {
                    QRScanner.destroy();
                }
            });
        }
    };

    this.backAndCancel = function (action, widget) {
        if (action) {
            action.isVisible = false;
        }
        widget.isScaning = false;
        document.body.style.background = '';
        if ('QRScanner' in window) {
            QRScanner.getStatus((status) => {
                if (status.scanning && status.showing) {
                    QRScanner.destroy();
                }
            });
        }
    };

    this.showQRCodeScanner = (widget) => {
        widget.isScaning = true;
        widget.actions.find(ac => ac.name === 'cancel').isVisible = false;
        if ('QRScanner' in window) {
            document.body.style.background = 'transparent';
            startScan().then((text) => {
                self.readQRCode(text)
                    .catch(() => this.showQRCodeScanner(widget))
                    .then(() => FCMBootstrap.start());
                document.body.style.background = '';
            }).catch((error) => {
                MessageService.showMessageByCode('MSG_INVALID_QRCODE', null, {type: 'error'});
                this.showQRCodeScanner(widget);
                document.body.style.background = '';
            });
            let fixBackground = () => {
                this.backAndCancelEvent(widget);
                window.removeEventListener('popstate', fixBackground);
            };
            window.addEventListener('popstate', fixBackground, false);
        }
    };
    

}

Configuration(function (ContextRegister) {
    ContextRegister.register('QRCodeController', QRCodeController);
});