/**
* Creates an instance of RequestFactory
* @class
* @name RequestFactory
* @param {TemplateManager} templateManager - An instance of TemplateManager
*/
function RequestFactory(templateManager) {
	var setDefaultProperties = function(source, desc) {
		desc.timeout = source.timeout;
		desc.retryCount = source.retryCount;
		desc.retryCallback = source.retryCallback;
	};
	/**
	* Creates and format a request to be sent
	* @function factory
	* @memberof RequestFactory
	* @instance
	* @param {Object} e - Object with the data to the request
	* @param {String} e.requestType - The type  of the request
	* @param {Number} [e.timeout ]- Time in milliseconds to the timout of the request
	* @param {Number} [e.retryCount] - Number of times to retry to send the request
	* @param {Function} [e.retryCallback] - Callback to be executed when retrying to send the request
	* @returns {Request} Returns a formated request
	*/
	this.factory = function(e) {
		var request = null;
		var currentContainer;
		var activeWidget;
		if(templateManager.container) {
			currentContainer = templateManager.container.name;
			activeWidget = templateManager.container.getActiveWidget();
			activeWidget = activeWidget ? activeWidget.name : undefined;
		}
		if (e.requestType) {
			switch (e.requestType) {
				case "filterData":
					request = new FilterDataRequest(e.serviceName, e.filter, e.callBack, e.page, e.errorCallBack, e.itemsPerPage, currentContainer, activeWidget);
					break;
				case "AutoCompleteData":
					request = new ReferenceAutoCompleteDataRequest(e.serviceName, e.value, e.inData, currentContainer, activeWidget);
					break;
				case "Row":
					request = new EventRequestRow(e.serviceName, e.row, e.callBack, currentContainer, activeWidget);
					break;
				case "File":
					request = new EventRequestFile(e.serviceName, e.row, e.callBack, currentContainer, activeWidget);
					break;
				case "CheckRow":
					request = new EventRequestCheckRow(e.serviceName, e.row, currentContainer, activeWidget);
					break;
				case "Cell":
					request = new EventCell(e.serviceName, e.value, currentContainer, activeWidget);
					break;
				case "dataSource":
					request = new EventRequestDataSet(e.serviceName, e.dataSource, e.callBack, currentContainer, activeWidget);
					break;
				case "Wizard":
					request = new EventWizard(e.serviceName, e.dataset, currentContainer, activeWidget);
					break;
				default:
					request = new EventEmpty(e.serviceName, currentContainer, activeWidget);
					break;
			}
		}
		setDefaultProperties(e, request);
		return request;
	};

	/**
	* @typedef {Object} Request
	* @property {String} requestType - The type of the request to be sent
	* @property {String} serviceName - The name of the service that will receive the request
	* @property {Function} processResponse - A function to be executed as a callback on a success response
	* @property {Function} processError - A function to be executed as a callback on a error response
	* @property {Number} timeout - Time in milliseconds to the timeout of the request
	* @property {Number} retryCount - Number of times to retry sending the request
	* @property {Function} retryCallback - Callback to be executed when retrying to send the request
	*/
}